/*  Copyright 2003-2016 Ciaran Anscomb
 *
 *  This file is part of XRoar.
 *
 *  XRoar is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  XRoar is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with XRoar.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <SDL.h>
#include <SDL_syswm.h>

#include "events.h"
#include "logging.h"
#include "machine.h"
#include "module.h"
#include "sam.h"
#include "ui.h"
#include "vo.h"
#include "xroar.h"

#include "sdl2/common.h"

/* Note: prefer the default order for sound and joystick modules, which
 * will include the SDL options. */

static _Bool init(void);
static void shutdown(void);
static void set_state(enum ui_tag tag, int value, const void *data);

struct ui_module ui_sdl_module = {
	.common = { .name = "sdl", .description = "SDL2 UI",
	            .init = init, .shutdown = shutdown },
	.vo_module_list = sdl_vo_module_list,
	.joystick_module_list = sdl_js_modlist,
	.run = sdl_run,
	.set_state = set_state,
};

static _Bool init(void) {

	if (!SDL_WasInit(SDL_INIT_NOPARACHUTE)) {
		if (SDL_Init(SDL_INIT_NOPARACHUTE) < 0) {
			LOG_ERROR("Failed to initialise SDL: %s\n", SDL_GetError());
			return 0;
		}
	}

	if (SDL_InitSubSystem(SDL_INIT_VIDEO) < 0) {
		LOG_ERROR("Failed to initialise SDL video: %s\n", SDL_GetError());
		return 0;
	}

#ifdef HAVE_X11
	SDL_EventState(SDL_SYSWMEVENT, SDL_ENABLE);
#endif

	sdl_keyboard_init();

	return 1;
}

static void shutdown(void) {
	SDL_QuitSubSystem(SDL_INIT_VIDEO);
}

static void set_state(enum ui_tag tag, int value, const void *data) {
	(void)data;
	switch (tag) {
	case ui_tag_kbd_translate:
		sdl_keyboard_set_translate(value);
		break;
	default:
		break;
	}
}
